package Pooru::API::V0::Controller::Tags;
use v5.40;
use Mojo::Base "Mojolicious::Controller";


sub list ($self)
{
	my $v = $self->validation;

	my $page = $v->optional("page")->num(1, undef)->param // 1;
	return $self->render(
		json => {error => "Invalid page number."},
		status => 400,
	) if $v->has_error;

	my $paged_tags = $self->tags_model->ranked->page($page);

	return $self->render(json => {
		tags => $paged_tags->fetchall,
		pager => $paged_tags->pager,
	});
}

sub show ($self)
{
	my $v = $self->validation;

	my $tag_id_or_name = $v->optional("id")->num(1, undef)->param //
	    $v->optional("display")->param //
	    $v->required("name")->param;
	return $self->render(
		json => {error => "Invalid tag ID or name."},
		status => 400,
	) if $v->has_error;

	my %search = ($v->topic => [$tag_id_or_name]);

	my $kind_id = $v->optional("kind_id")->param;
	return $self->render(
		json => {error => "Invalid kind ID."},
		status => 400,
	) if $v->has_error;
	$search{kind_id} = $kind_id if defined($kind_id);

	my @tag_ids = map {$_->{id}} $self->tags_model
	    ->get(%search)->fetchall->@*;
	my @tags = $self->tags_model->ranked_for_tags(@tag_ids)->fetchall->@*;

	return $self->render(
		json => {error => "Tag not found."},
		status => 404,
	) if @tags == 0;

	return $self->render(json => {tags => [@tags]});
}

sub random ($self)
{
	my $id = $self->tags_model->random_id;

	return $self->render(
		json => {error => "Tag not found."},
		status => 404,
	) if !defined($id);

	return $self->redirect_to(
	    $self->url_for("show_tag")->query(id => $id));
}

sub search ($self)
{
	my $v = $self->validation;

	my $q = $v->required("q")->param;
	return $self->render(
		json => {error => "Missing query."},
		status => 400,
	) if $v->has_error;

	return $self->render(
		json => {tags => []},
		status => 200
	) if $q eq "";

	$q =~ s/[%\\_]/\\$&/g;
	$q .= '%';
	my @tags = $self->tags_model->search($q)->fetchall->@*;

	return $self->render(json => {tags => [@tags]});
}

1;
